==> tutti-ui-swing-3.6-sources.jar: fr/ifremer/tutti/ui/swing/content/operation/catches/benthos/BenthosBatchTableModel.java <==
package fr.ifremer.tutti.ui.swing.content.operation.catches.benthos;

/*
 * #%L
 * Tutti :: UI
 * %%
 * Copyright (C) 2012 - 2014 Ifremer
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-3.0.html>.
 * #L%
 */

import com.google.common.base.Preconditions;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import org.nuiton.jaxx.application.swing.table.AbstractApplicationTableModel;
import fr.ifremer.tutti.persistence.entities.data.SampleCategory;
import fr.ifremer.tutti.persistence.entities.data.SampleCategoryModel;
import fr.ifremer.tutti.persistence.entities.referential.Species;
import fr.ifremer.tutti.type.WeightUnit;
import fr.ifremer.tutti.ui.swing.content.operation.catches.SampleCategoryColumnIdentifier;
import org.nuiton.jaxx.application.swing.table.ColumnIdentifier;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdesktop.swingx.table.TableColumnModelExt;

import javax.swing.table.TableColumn;
import java.io.Serializable;
import java.util.Enumeration;
import java.util.List;
import java.util.Set;

import static org.nuiton.i18n.I18n.n;

/**
 * @author tchemit <chemit@codelutin.com>
 * @since 0.1
 */
public class BenthosBatchTableModel extends AbstractApplicationTableModel<BenthosBatchRowModel> {

    private static final long serialVersionUID = 1L;

    /** Logger. */
    private static final Log log =
            LogFactory.getLog(BenthosBatchTableModel.class);

    public static final ColumnIdentifier<BenthosBatchRowModel> SPECIES = ColumnIdentifier.newId(
            BenthosBatchRowModel.PROPERTY_SPECIES_ROW,
            n("tutti.editBenthosBatch.table.header.species"),
            n("tutti.editBenthosBatch.table.header.species.tip"));

    public static final ColumnIdentifier<BenthosBatchRowModel> WEIGHT = ColumnIdentifier.newId(
            BenthosBatchRowModel.PROPERTY_COMPUTED_WEIGHT,
            n("tutti.editBenthosBatch.table.header.weight"),
            n("tutti.editBenthosBatch.table.header.weight.tip"));

    public static final ColumnIdentifier<BenthosBatchRowModel> COMPUTED_NUMBER = ColumnIdentifier.newId(
            BenthosBatchRowModel.PROPERTY_COMPUTED_NUMBER,
            n("tutti.editBenthosBatch.table.header.computedNumber"),
            n("tutti.editBenthosBatch.table.header.computedNumber.tip"));

    public static final ColumnIdentifier<BenthosBatchRowModel> COMMENT = ColumnIdentifier.newId(
            BenthosBatchRowModel.PROPERTY_COMMENT,
            n("tutti.editBenthosBatch.table.header.comment"),
            n("tutti.editBenthosBatch.table.header.comment.tip"));

    public static final ColumnIdentifier<BenthosBatchRowModel> ATTACHMENT = ColumnIdentifier.newReadOnlyId(
            BenthosBatchRowModel.PROPERTY_ATTACHMENT,
            n("tutti.editBenthosBatch.table.header.file"),
            n("tutti.editBenthosBatch.table.header.file.tip"));

    public static final ColumnIdentifier<BenthosBatchRowModel> SPECIES_TO_CONFIRM = ColumnIdentifier.newId(
            BenthosBatchRowModel.PROPERTY_SPECIES_TO_CONFIRM,
            n("tutti.editBenthosBatch.table.header.toConfirm"),
            n("tutti.editBenthosBatch.table.header.toConfirm.tip"));

    /**
     * Columns for the frequency edition.
     *
     * @since 0.2
     */
    protected final Set<ColumnIdentifier<BenthosBatchRowModel>> frequencyCols;

    /**
     * Columns implies in the sample category definition.
     *
     * @since 0.2
     */
    protected final Set<SampleCategoryColumnIdentifier<BenthosBatchRowModel>> sampleCols;

    /**
     * Sample categories model.
     *
     * @since 2.4
     */
    protected final SampleCategoryModel sampleCategoryModel;

    /**
     * Weight unit.
     *
     * @since 2.5
     */
    protected final WeightUnit weightUnit;

    public BenthosBatchTableModel(WeightUnit weightUnit,
                                  SampleCategoryModel sampleCategoryModel,
                                  TableColumnModelExt columnModel) {
        super(columnModel, false, false);
        this.weightUnit = weightUnit;
        this.sampleCategoryModel = sampleCategoryModel;
        setNoneEditableCols(SPECIES);

        frequencyCols = Sets.newHashSet();
        frequencyCols.add(COMPUTED_NUMBER);
        frequencyCols.add(WEIGHT);

        sampleCols = Sets.newHashSet();
        Enumeration<TableColumn> columns = columnModel.getColumns();
        while (columns.hasMoreElements()) {
            TableColumn tableColumn = columns.nextElement();
            Object identifier = tableColumn.getIdentifier();
            if (identifier instanceof SampleCategoryColumnIdentifier<?>) {
                SampleCategoryColumnIdentifier<BenthosBatchRowModel> categoryColumnIdentifier =
                        (SampleCategoryColumnIdentifier<BenthosBatchRowModel>) identifier;
                sampleCols.add(categoryColumnIdentifier);
            }
        }
    }

    @Override
    protected void collectShell(BenthosBatchRowModel row,
                                Set<BenthosBatchRowModel> collectedRows) {
        super.collectShell(row, collectedRows);

        if (!row.isChildBatchsEmpty()) {
            for (BenthosBatchRowModel child : row.getChildBatch()) {
                collectShell(child, collectedRows);
            }
        }
    }

    @Override
    public BenthosBatchRowModel createNewRow() {
        BenthosBatchRowModel result =
                new BenthosBatchRowModel(weightUnit, sampleCategoryModel);

        // by default empty row is not valid
        result.setValid(false);
        return result;
    }

    @Override
    public void setValueAt(Object aValue,
                           int rowIndex,
                           int columnIndex,
                           ColumnIdentifier<BenthosBatchRowModel> propertyName,
                           BenthosBatchRowModel entry) {
        if (sampleCols.contains(propertyName)) {

            SampleCategoryColumnIdentifier<BenthosBatchRowModel> sampleCategoryColumnIdentifier = (SampleCategoryColumnIdentifier<BenthosBatchRowModel>) propertyName;
            sampleCategoryColumnIdentifier.setWeightValue(entry, aValue);

            // must find out first ancestor with this category
            Integer sampleCategoryId = sampleCategoryColumnIdentifier.getSampleCategoryId();
            if (log.isDebugEnabled()) {
                log.debug("Sample category: " + sampleCategoryId + " modified at row: " + rowIndex);
            }
            SampleCategory<?> sampleCategory = entry.getSampleCategoryById(sampleCategoryId);
            BenthosBatchRowModel firstAncestor = entry.getFirstAncestor(sampleCategory);

            int firstRowIndex = getRowIndex(firstAncestor);
            if (log.isDebugEnabled()) {
                log.debug("First ancestor row: " + firstRowIndex);
            }

            // save this row and his shell
            updateShell(firstAncestor, columnIndex);

        } else {
            super.setValueAt(aValue, rowIndex, columnIndex, propertyName, entry);
        }
    }

    @Override
    protected boolean isCellEditable(int rowIndex,
                                     int columnIndex,
                                     ColumnIdentifier<BenthosBatchRowModel> propertyName) {

        boolean result = super.isCellEditable(rowIndex,
                                              columnIndex,
                                              propertyName);
        if (result) {

            if (frequencyCols.contains(propertyName)) {

                // must have filled a species to edit this column
                BenthosBatchRowModel entry = getEntry(rowIndex);
                result = entry.isBatchLeaf();

            } else if (sampleCols.contains(propertyName)) {

                // can only edit if a category value is setted
                BenthosBatchRowModel entry = getEntry(rowIndex);

                // check from protocol what is possible ?
                Species species = entry.getSpecies();

                if (species == null) {

                    // no species, can not edit
                    result = false;
                } else {

                    // protocol authorize it

                    // final test: can edit only if sample category is setted
                    SampleCategory<?> value =
                            (SampleCategory<?>) propertyName.getValue(entry);

                    result = value.getCategoryValue() != null;

                }
            }
        }
        return result;
    }

    /**
     * Return the next editable row index for frequency from the given
     * {@code rowIndex}, or {@code null} if none found.
     *
     * @param rowIndex the starting index where to look
     * @return the next editable row index for frequency from the given {@code rowIndex}, or {@code null} if none found.
     * @since 2.5
     */
    public Integer getNextEditableFrequencyRow(Integer rowIndex) {
        Integer result = null;

        for (int i = rowIndex, max = getRowCount(); i < max; i++) {
            BenthosBatchRowModel entry = getEntry(i);
            if (entry.isBatchLeaf()) {
                result = i;
                break;
            }
        }
        return result;
    }

    /**
     * Return the index of the next row that can be inserted as a child of the
     * given row.
     *
     * @param row the parent row
     * @return the index of the next row that can be inserted as a child of the
     * given row
     * @since 2.7
     */
    public final int getNextChildRowIndex(BenthosBatchRowModel row) {

        // get row index of the given row
        int parentRowIndex = getRowIndex(row);

        // must find it
        Preconditions.checkState(parentRowIndex != -1);

        int result = parentRowIndex;

        if (!row.isChildBatchsEmpty()) {

            // take his last child
            BenthosBatchRowModel lastChild =
                    row.getChildBatch().get(row.sizeChildBatchs() - 1);

            // get the shell of this child (including himself)
            Set<BenthosBatchRowModel> childs = Sets.newHashSet();
            childs.add(lastChild);
            lastChild.collectShell(childs);

            // get row with max index
            for (BenthosBatchRowModel child : childs) {
                int childRowIndex = getRowIndex(child);
                result = Math.max(childRowIndex, result);
            }
        }

        // result is the nex row
        result++;

        return result;
    }

    /**
     * Return the sample category id of a column or {@code null} if not on a
     * sample category column.
     *
     * @param columnIndex index of the column to look at
     * @return the sample category id of a column or {@code null} if not on a
     * sample category column.
     * @since 2.6
     */
    public Integer getSampleCategoryId(int columnIndex) {
        Integer result = null;
        ColumnIdentifier<BenthosBatchRowModel> identifier = getIdentifier(columnIndex);
        if (sampleCols.contains(identifier)) {
            SampleCategoryColumnIdentifier<BenthosBatchRowModel> sampleId = (SampleCategoryColumnIdentifier<BenthosBatchRowModel>) identifier;
            result = sampleId.getSampleCategoryId();
        }
        return result;
    }

    /**
     * Update the sample category value of the given {@code row}.
     *
     * @param row         the row to walk through
     * @param columnIndex index of the column where the sample category is
     * @param newValue    new sample category value to set
     * @since 2.6
     */
    public void updateSampleCategorieValue(BenthosBatchRowModel row,
                                           int columnIndex,
                                           Serializable newValue) {

        SampleCategoryColumnIdentifier<BenthosBatchRowModel> identifier =
                (SampleCategoryColumnIdentifier<BenthosBatchRowModel>) getIdentifier(columnIndex);

        // set category value
        identifier.setCategoryValue(row, newValue);

        // update row shell
        updateShell(row, columnIndex);
    }

    /**
     * Return previous sibling row, or {@code null} if not exist.
     *
     * @param row where to start
     * @return the previous sibling row, or {@code null} if not exist.
     * @since 2.6
     */
    public BenthosBatchRowModel getPreviousSibling(BenthosBatchRowModel row) {
        BenthosBatchRowModel result = null;
        BenthosBatchRowModel parentBatch = row.getParentBatch();
        if (parentBatch != null) {
            List<BenthosBatchRowModel> childBatch = parentBatch.getChildBatch();
            int i = childBatch.indexOf(row);
            if (i > 0) {
                result = childBatch.get(i - 1);
            }
        }
        return result;
    }

    /**
     * Update all the given cells for the  given column.
     *
     * @param shell       shell to update
     * @param columnIndex the column index to update
     * @since 2.6
     */
    public void updateShell(Set<BenthosBatchRowModel> shell, int columnIndex) {
        for (BenthosBatchRowModel batchRowModel : shell) {
            int currentRowIndex = getRowIndex(batchRowModel);
            if (log.isDebugEnabled()) {
                log.debug("Update shell row: " + currentRowIndex);
            }
            fireTableCellUpdated(currentRowIndex, columnIndex);
        }
    }

    /**
     * Return the list of used species in the table (used to fill the
     * comparator cache for species sort)
     *
     * @return the list of used species in the table.
     * @since 2.8
     */
    public List<Species> getSpeciesList() {
        List<Species> result = Lists.newArrayList();
        for (BenthosBatchRowModel row : rows) {
            if (row.isBatchRoot()) {
                result.add(row.getSpecies());
            }
        }
        return result;
    }

    /**
     * Update all the cells of the given row shell for a given column.
     *
     * @param entry       the first row to update
     * @param columnIndex the column index to update
     * @since 2.6
     */
    protected void updateShell(BenthosBatchRowModel entry, int columnIndex) {

        Set<BenthosBatchRowModel> shell = Sets.newHashSet();
        entry.collectShell(shell);
        shell.add(entry);
        updateShell(shell, columnIndex);
    }
}

==> tutti-ui-swing-3.8-sources.jar: fr/ifremer/tutti/ui/swing/content/operation/catches/benthos/BenthosBatchTableModel.java <==
package fr.ifremer.tutti.ui.swing.content.operation.catches.benthos;

/*
 * #%L
 * Tutti :: UI
 * %%
 * Copyright (C) 2012 - 2014 Ifremer
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-3.0.html>.
 * #L%
 */

import com.google.common.base.Preconditions;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import fr.ifremer.tutti.persistence.entities.data.SampleCategory;
import fr.ifremer.tutti.persistence.entities.data.SampleCategoryModel;
import fr.ifremer.tutti.persistence.entities.referential.Species;
import fr.ifremer.tutti.type.WeightUnit;
import fr.ifremer.tutti.ui.swing.content.operation.catches.SampleCategoryColumnIdentifier;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdesktop.swingx.table.TableColumnModelExt;
import org.nuiton.jaxx.application.swing.table.AbstractApplicationTableModel;
import org.nuiton.jaxx.application.swing.table.ColumnIdentifier;

import javax.swing.table.TableColumn;
import java.io.Serializable;
import java.util.Enumeration;
import java.util.List;
import java.util.Set;

import static org.nuiton.i18n.I18n.n;

/**
 * @author tchemit <chemit@codelutin.com>
 * @since 0.1
 */
public class BenthosBatchTableModel extends AbstractApplicationTableModel<BenthosBatchRowModel> {

    private static final long serialVersionUID = 1L;

    /** Logger. */
    private static final Log log =
            LogFactory.getLog(BenthosBatchTableModel.class);

    public static final ColumnIdentifier<BenthosBatchRowModel> SPECIES = ColumnIdentifier.newId(
            BenthosBatchRowModel.PROPERTY_SPECIES_ROW,
            n("tutti.editBenthosBatch.table.header.species"),
            n("tutti.editBenthosBatch.table.header.species.tip"));

    public static final ColumnIdentifier<BenthosBatchRowModel> WEIGHT = ColumnIdentifier.newId(
            BenthosBatchRowModel.PROPERTY_COMPUTED_WEIGHT,
            n("tutti.editBenthosBatch.table.header.weight"),
            n("tutti.editBenthosBatch.table.header.weight.tip"));

    public static final ColumnIdentifier<BenthosBatchRowModel> COMPUTED_NUMBER = ColumnIdentifier.newId(
            BenthosBatchRowModel.PROPERTY_COMPUTED_NUMBER,
            n("tutti.editBenthosBatch.table.header.computedNumber"),
            n("tutti.editBenthosBatch.table.header.computedNumber.tip"));

    public static final ColumnIdentifier<BenthosBatchRowModel> COMMENT = ColumnIdentifier.newId(
            BenthosBatchRowModel.PROPERTY_COMMENT,
            n("tutti.editBenthosBatch.table.header.comment"),
            n("tutti.editBenthosBatch.table.header.comment.tip"));

    public static final ColumnIdentifier<BenthosBatchRowModel> ATTACHMENT = ColumnIdentifier.newReadOnlyId(
            BenthosBatchRowModel.PROPERTY_ATTACHMENT,
            n("tutti.editBenthosBatch.table.header.file"),
            n("tutti.editBenthosBatch.table.header.file.tip"));

    public static final ColumnIdentifier<BenthosBatchRowModel> SPECIES_TO_CONFIRM = ColumnIdentifier.newId(
            BenthosBatchRowModel.PROPERTY_SPECIES_TO_CONFIRM,
            n("tutti.editBenthosBatch.table.header.toConfirm"),
            n("tutti.editBenthosBatch.table.header.toConfirm.tip"));

    /**
     * Columns for the frequency edition.
     *
     * @since 0.2
     */
    protected final Set<ColumnIdentifier<BenthosBatchRowModel>> frequencyCols;

    /**
     * Columns implies in the sample category definition.
     *
     * @since 0.2
     */
    protected final Set<SampleCategoryColumnIdentifier<BenthosBatchRowModel>> sampleCols;

    /**
     * Sample categories model.
     *
     * @since 2.4
     */
    protected final SampleCategoryModel sampleCategoryModel;

    /**
     * Weight unit.
     *
     * @since 2.5
     */
    protected final WeightUnit weightUnit;

    public BenthosBatchTableModel(WeightUnit weightUnit,
                                  SampleCategoryModel sampleCategoryModel,
                                  TableColumnModelExt columnModel) {
        super(columnModel, false, false);
        this.weightUnit = weightUnit;
        this.sampleCategoryModel = sampleCategoryModel;
        setNoneEditableCols(SPECIES);

        frequencyCols = Sets.newHashSet();
        frequencyCols.add(COMPUTED_NUMBER);
        frequencyCols.add(WEIGHT);

        sampleCols = Sets.newHashSet();
        Enumeration<TableColumn> columns = columnModel.getColumns();
        while (columns.hasMoreElements()) {
            TableColumn tableColumn = columns.nextElement();
            Object identifier = tableColumn.getIdentifier();
            if (identifier instanceof SampleCategoryColumnIdentifier<?>) {
                SampleCategoryColumnIdentifier<BenthosBatchRowModel> categoryColumnIdentifier =
                        (SampleCategoryColumnIdentifier<BenthosBatchRowModel>) identifier;
                sampleCols.add(categoryColumnIdentifier);
            }
        }
    }

    @Override
    protected void collectShell(BenthosBatchRowModel row,
                                Set<BenthosBatchRowModel> collectedRows) {
        super.collectShell(row, collectedRows);

        if (!row.isChildBatchsEmpty()) {
            for (BenthosBatchRowModel child : row.getChildBatch()) {
                collectShell(child, collectedRows);
            }
        }
    }

    @Override
    public BenthosBatchRowModel createNewRow() {
        BenthosBatchRowModel result =
                new BenthosBatchRowModel(weightUnit, sampleCategoryModel);

        // by default empty row is not valid
        result.setValid(false);
        return result;
    }

    @Override
    public void setValueAt(Object aValue,
                           int rowIndex,
                           int columnIndex,
                           ColumnIdentifier<BenthosBatchRowModel> propertyName,
                           BenthosBatchRowModel entry) {
        if (sampleCols.contains(propertyName)) {

            SampleCategoryColumnIdentifier<BenthosBatchRowModel> sampleCategoryColumnIdentifier = (SampleCategoryColumnIdentifier<BenthosBatchRowModel>) propertyName;
            sampleCategoryColumnIdentifier.setWeightValue(entry, aValue);

            // must find out first ancestor with this category
            Integer sampleCategoryId = sampleCategoryColumnIdentifier.getSampleCategoryId();
            if (log.isDebugEnabled()) {
                log.debug("Sample category: " + sampleCategoryId + " modified at row: " + rowIndex);
            }
            SampleCategory<?> sampleCategory = entry.getSampleCategoryById(sampleCategoryId);
            BenthosBatchRowModel firstAncestor = entry.getFirstAncestor(sampleCategory);

            int firstRowIndex = getRowIndex(firstAncestor);
            if (log.isDebugEnabled()) {
                log.debug("First ancestor row: " + firstRowIndex);
            }

            // save this row and his shell
            updateShell(firstAncestor, columnIndex);

        } else {
            super.setValueAt(aValue, rowIndex, columnIndex, propertyName, entry);
        }
    }

    @Override
    protected boolean isCellEditable(int rowIndex,
                                     int columnIndex,
                                     ColumnIdentifier<BenthosBatchRowModel> propertyName) {

        boolean result = super.isCellEditable(rowIndex,
                                              columnIndex,
                                              propertyName);
        if (result) {

            if (frequencyCols.contains(propertyName)) {

                // must have filled a species to edit this column
                BenthosBatchRowModel entry = getEntry(rowIndex);
                result = entry.isBatchLeaf();

            } else if (sampleCols.contains(propertyName)) {

                // can only edit if a category value is setted
                BenthosBatchRowModel entry = getEntry(rowIndex);

                // check from protocol what is possible ?
                Species species = entry.getSpecies();

                if (species == null) {

                    // no species, can not edit
                    result = false;
                } else {

                    // protocol authorize it

                    // final test: can edit only if sample category is setted
                    SampleCategory<?> value =
                            (SampleCategory<?>) propertyName.getValue(entry);

                    result = value.getCategoryValue() != null;

                }
            }
        }
        return result;
    }

    /**
     * Return the next editable row index for frequency from the given
     * {@code rowIndex}, or {@code null} if none found.
     *
     * @param rowIndex the starting index where to look
     * @return the next editable row index for frequency from the given {@code rowIndex}, or {@code null} if none found.
     * @since 2.5
     */
    public Integer getNextEditableFrequencyRow(Integer rowIndex) {
        Integer result = null;

        for (int i = rowIndex, max = getRowCount(); i < max; i++) {
            BenthosBatchRowModel entry = getEntry(i);
            if (entry.isBatchLeaf()) {
                result = i;
                break;
            }
        }
        return result;
    }

    /**
     * Return the index of the next row that can be inserted as a child of the
     * given row.
     *
     * @param row the parent row
     * @return the index of the next row that can be inserted as a child of the
     * given row
     * @since 2.7
     */
    public final int getNextChildRowIndex(BenthosBatchRowModel row) {

        // get row index of the given row
        int parentRowIndex = getRowIndex(row);

        // must find it
        Preconditions.checkState(parentRowIndex != -1);

        int result = parentRowIndex;

        if (!row.isChildBatchsEmpty()) {

            // take his last child
            BenthosBatchRowModel lastChild =
                    row.getChildBatch().get(row.sizeChildBatchs() - 1);

            // get the shell of this child (including himself)
            Set<BenthosBatchRowModel> childs = Sets.newHashSet();
            childs.add(lastChild);
            lastChild.collectShell(childs);

            // get row with max index
            for (BenthosBatchRowModel child : childs) {
                int childRowIndex = getRowIndex(child);
                result = Math.max(childRowIndex, result);
            }
        }

        // result is the nex row
        result++;

        return result;
    }

    /**
     * Return the sample category id of a column or {@code null} if not on a
     * sample category column.
     *
     * @param columnIndex index of the column to look at
     * @return the sample category id of a column or {@code null} if not on a
     * sample category column.
     * @since 2.6
     */
    public Integer getSampleCategoryId(int columnIndex) {
        Integer result = null;
        ColumnIdentifier<BenthosBatchRowModel> identifier = getIdentifier(columnIndex);
        if (sampleCols.contains(identifier)) {
            SampleCategoryColumnIdentifier<BenthosBatchRowModel> sampleId = (SampleCategoryColumnIdentifier<BenthosBatchRowModel>) identifier;
            result = sampleId.getSampleCategoryId();
        }
        return result;
    }

    /**
     * Update the sample category value of the given {@code row}.
     *
     * @param row         the row to walk through
     * @param columnIndex index of the column where the sample category is
     * @param newValue    new sample category value to set
     * @return the old category value replaced by the new one
     * @since 2.6
     */
    public Serializable updateSampleCategorieValue(BenthosBatchRowModel row,
                                                   int columnIndex,
                                                   Serializable newValue) {

        SampleCategoryColumnIdentifier<BenthosBatchRowModel> identifier =
                (SampleCategoryColumnIdentifier<BenthosBatchRowModel>) getIdentifier(columnIndex);

        Serializable oldValue = identifier.getCategoryValue(row);

        // set category value
        identifier.setCategoryValue(row, newValue);

        // update row shell
        updateShell(row, columnIndex);

        return oldValue;

    }

    /**
     * Return previous sibling row, or {@code null} if not exist.
     *
     * @param row where to start
     * @return the previous sibling row, or {@code null} if not exist.
     * @since 2.6
     */
    public BenthosBatchRowModel getPreviousSibling(BenthosBatchRowModel row) {
        BenthosBatchRowModel result = null;
        BenthosBatchRowModel parentBatch = row.getParentBatch();
        if (parentBatch != null) {
            List<BenthosBatchRowModel> childBatch = parentBatch.getChildBatch();
            int i = childBatch.indexOf(row);
            if (i > 0) {
                result = childBatch.get(i - 1);
            }
        }
        return result;
    }

    /**
     * Update all the given cells for the  given column.
     *
     * @param shell       shell to update
     * @param columnIndex the column index to update
     * @since 2.6
     */
    public void updateShell(Set<BenthosBatchRowModel> shell, int columnIndex) {
        for (BenthosBatchRowModel batchRowModel : shell) {
            int currentRowIndex = getRowIndex(batchRowModel);
            if (log.isDebugEnabled()) {
                log.debug("Update shell row: " + currentRowIndex);
            }
            fireTableCellUpdated(currentRowIndex, columnIndex);
        }
    }

    /**
     * Return the list of used species in the table (used to fill the
     * comparator cache for species sort)
     *
     * @return the list of used species in the table.
     * @since 2.8
     */
    public List<Species> getSpeciesList() {
        List<Species> result = Lists.newArrayList();
        for (BenthosBatchRowModel row : rows) {
            if (row.isBatchRoot()) {
                result.add(row.getSpecies());
            }
        }
        return result;
    }

    /**
     * Update all the cells of the given row shell for a given column.
     *
     * @param entry       the first row to update
     * @param columnIndex the column index to update
     * @since 2.6
     */
    protected void updateShell(BenthosBatchRowModel entry, int columnIndex) {

        Set<BenthosBatchRowModel> shell = Sets.newHashSet();
        entry.collectShell(shell);
        shell.add(entry);
        updateShell(shell, columnIndex);
    }
}
